/**
 * Module dependencies.
 */

var _ = require('lodash');

/**
 * @param  {SailsApp} sails  [Sails app]
 * @return {Function}
 */
module.exports = function to(sails) {

  /**
   * `interpretRouteSyntax()`
   *
   * "Teach" router to understand references to actions.
   * This is the event handler for the 'route:typeUnknown' emitted on `sails`.
   *
   * @param  {Dictionary} route
   *         @property {String} verb
   *         @property {String} path
   *         @property {Dictionary} target
   *         @property {Dictionary} options
   *
   * @api private
   */
  return function interpretRouteSyntax (route) {
    var target = route.target;
    var path = route.path;
    var verb = route.verb;
    var options = route.options;


    // Support various dictionary target notations, e.g.:
    // `{ controller: 'UserController' }`
    if (_.isObject(target) && !_.isFunction(target) && !_.isArray(target)) {

      // Merge target into `options` to get hold of relevant route options:
      options = _.merge(options, target);
      // Note: this (^) could be moved up into lib/router/bind.js, since its
      // only pertinent for core options such as `skipAssets`.  There would need
      // to be changes in other hooks as well.

      // We only care about the syntax that has `action` all by itself
      if ( !_.isUndefined(target.action) && _.keys(target).length === 1 ) {

        var action = sails.actions[target.action];
        // If there is no action w/ the given identity, log a warning
        // and continue.
        if (!action) {
          sails.log.warn('Ignoring attempt to bind route to unknown action `' + target.action + '`');
          return;
        }

        // If there is an action but it's not a function, bail hard.
        if (!_.isFunction(action) || action.name !== '_requestHandler') {
          throw new Error('Attempted to bind route to action `' + target.action + '`, but it turned out not to be a valid Sails action machine (i.e. something returned by `machine-as-action`)');
        }

        // Bind the route to the action
        return sails.router.bind(path, action, verb, options);

      }

    }

    // Ignore unknown route syntax
    // If it needs to be understood by another hook, the hook would have also received
    // the typeUnknown event, so we're done.
    return;
  };


};

